// Licensed Materials - Property of IBM
// 5648-F10 (C) Copyright International Business Machines Corp. 2005, 2006
// All Rights Reserved
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

// array that maps disk IDs to real disk directories
top.diskMappings = new Array();
// array of all valid disk labels
top.allLabels = new Array();
// starting disk label
top.diskLabel = new Array();

var DISK_STRUCTURE_PROPERTIES = "diskStructure.properties";
var WRONG_DISK_ID = "wrongDiskId";
var NOT_DEFINED = "-NULL-";

// determine if 2 labels are compatible
// diskLabel: string - anchor label id for comparing
// compatibleLabel: string - label id to be tested
// returns boolean - if the 2 labels are compatible
function isCompatibleDiskLabel(diskLabel, compatibleLabel) {
    try {
        var compatibleChecks = compatibleLabel["compatible"];
        if ((typeof compatibleChecks) == "undefined" || compatibleChecks == null) return true;
        var isCompatible = true;
        for (var i in compatibleChecks) try
        {
            var v = '';
            with (diskLabel) {
              //For complex ids, split the id on the plus sign and evaluate each part seperately
              var indexArray = i.split("+");
              for (var index in indexArray) {
                var tempValue = "";
                try { tempValue = eval(indexArray[index]); } 
                catch(e) { tempValue = NOT_DEFINED; }
                v += tempValue;                
              }              
            }

            var regexp = new RegExp("^(" + compatibleChecks[i] + ")$","i");
            var m = v.match(regexp);
            if (m != null && (typeof m != "string")) m = m[0];

            isCompatible = (m == v);
            if (!isCompatible) return isCompatible;
        } catch(e) {
        }
        return isCompatible;
    } catch(e) {
        top.logException(e,arguments);
    }
    return top.UNDEFINED;
}

//Clean this up... it's got tons of duplicate code
function getIncompatibleMessage(diskLabel, compatibleLabel) {
    try {
        var retVal = "";
        var compatibleChecks = diskLabel["compatible"];
        if ((typeof compatibleChecks) == "undefined" || compatibleChecks == null) return top.UNDEFINED;
        var isCompatible = true;
        for (var i in compatibleChecks) try
        {
            if (isCompatible) {
                var v = '';
                // variables not found are assumed to be disklabel indexes
                try { v = eval(i);} catch(e) {
                    v = compatibleLabel[i];
                }
                var regexp = new RegExp(compatibleChecks[i],"i");
                var m = v.match(regexp);
                if (m != null && (typeof m != "string")) m = m[0];
                isCompatible = (m == v);
                if (!isCompatible) {
                    //If a launchpad variable failed a compatibility check, remove the "top." so
                    //the message is more user friendly
                    if (i.length > 4 && i.substring(0,4) == "top.") {
                        i = i.substring(4, i.length);
                    }
                    var message = top.formatmsg(document.properties['diskIncompatibilityDetails'], v, i);
                    retVal = message;
                }
            }
        } catch(e) {
        }
        return retVal;
    } catch(e) {
        top.logException(e,arguments);
    }
    return top.UNDEFINED;
}

// convert disk ID to real disk directory - this will prompt the user if needed
// id: string - the generic disk ID
// returns string - the real disk directory
function getDiskMapping(id) {
    var WIDTH = 410;
    var HEIGHT = 170;
    try {
        if (typeof id == "undefined") return null;
        if (id == null) {
            return top.diskMappings[diskLabel.id];
        }
        if (typeof top.allLabels[id] == "undefined" || top.allLabels[id] == null) {
            top.logMessage("LPV20015S", id);
            return top.UNDEFINED;
        }

        var diskMapping = top.diskMappings[id];

        if (typeof diskMapping != "undefined" 
            && top.secureFileExists(new Function('return window'), top.getNativeFileName(diskMapping+'/'+top.RELATIVEDIR+top.DISKINFODIR+top.THISDISKINFO))) {
            var diskLabelProperties = new Array();
            top.secureRead1PropertyFile(new Function('return window'), top.getNativeFileName(diskMapping+'/'+top.RELATIVEDIR+top.DISKINFODIR+top.THISDISKINFO), diskLabelProperties, false);
            if (diskLabelProperties.id == id && isCompatibleDiskLabel(diskLabelProperties,top.diskLabel)) {
                return diskMapping;
            }
        }

        var tempLabel = top.allLabels[id];
        if (typeof tempLabel == "object") {
            tempLabel = top.getBestOSMatch(tempLabel);            
        }
        var message = top.formatmsg(document.properties['diskDirPrompt'], tempLabel);
        var showWrongDirectory = false;
        
        if ((typeof diskMapping == "undefined") || diskMapping == null)
            diskMapping = top.STARTINGDIR;

        do {

            //set the default error message
            tempLabel = top.allLabels[id];
            if (typeof tempLabel == "object") {
                tempLabel = top.getBestOSMatch(tempLabel);            
            }
            message = top.formatmsg(document.properties['diskDirPrompt'], tempLabel);

            if (top.secureFileExists(new Function('return window'), top.getNativeFileName(diskMapping+'/'+top.RELATIVEDIR+top.DISKINFODIR+top.THISDISKINFO)) 
                || top.secureFileExists(new Function('return window'), top.getNativeFileName(diskMapping+'/'+DISK_STRUCTURE_PROPERTIES))) {
                var mediaTop = findMediaTop(top.getNativeFileName(diskMapping + "../"), top.getNativeFileName(diskMapping + "/"));
                var errorString = updateDiskMappings(mediaTop);
                var diskLabelProperties = new Array();
                if (typeof top.diskMappings[id] != "undefined") {
                    top.secureRead1PropertyFile(new Function('return window'), top.getNativeFileName(top.diskMappings[id]+'/'+top.RELATIVEDIR+top.DISKINFODIR+top.THISDISKINFO), diskLabelProperties, false);
                    if (diskLabelProperties.id == id && isCompatibleDiskLabel(diskLabelProperties,top.diskLabel)) {
                        diskMapping=top.diskMappings[id];
                        break;
                    }
                } else {
                    if (typeof errorString != "undefined" && errorString != WRONG_DISK_ID && errorString != "") {
                        message = top.formatmsg(document.properties['incompatibleDiskPrompt'], errorString);                   
                    }
                    //if we get here and there's no error string, it's probably because the user was prompted
                    //for a CD and never inserted the correct one.  An incompatibleDiskPrompt with a null error string is
                    //a pretty useless message, so we should fall back to the diskDirPrompt.
                    else if (errorString == "") {
                        tempLabel = top.allLabels[id];
                        if (typeof tempLabel == "object") {
                            tempLabel = top.getBestOSMatch(tempLabel);            
                        }

                        message = top.formatmsg(document.properties['diskDirPrompt'], tempLabel);
                    }
                }
            } 
            //If we come through here a 2nd time, it means that the last directory entered was invalid
            if (showWrongDirectory) {
                var wrongDirMessage = top.formatmsg(document.properties['wrongDirectory'], diskMapping);
                alert(wrongDirMessage);
            }
            else {
                showWrongDirectory = true;
            }

            diskMapping = top.modalPromptWithBrowse(property('promptTitle'), message ,diskMapping, top.isDirOK, WIDTH, HEIGHT);

            // Add '/' at end if not provided
            if (diskMapping != null && 
                (diskMapping.charAt(diskMapping.length-1) != "/") &&
                (diskMapping.charAt(diskMapping.length-1) != "\\"))
                diskMapping=diskMapping+"/";

        } while (diskMapping != null);
        return diskMapping;
    } catch(e) {
        top.logException(e,arguments);
    }
    return top.UNDEFINED;
}

//Searches for other disks that may exist in a DVD and updates the disk mapping array
//with their location.
//
//directory: string - The top level directory where the recursive search begins
function updateDiskMappings(directory) {
    var errorString="";

    try{        
        var diskStructureFile = top.getNativeFileName(directory + "/" + DISK_STRUCTURE_PROPERTIES);
        if (top.secureFileExists(new Function('return window'), diskStructureFile)) {
            var diskInfo = new Array();    
            top.secureRead1PropertyFile(new Function('return window'), diskStructureFile, diskInfo, false);
            if (typeof diskInfo['recurse'] != "undefined") {
                for (var i = 0; i < diskInfo['recurse'].length; i++) {
                    var recurseDirectory = top.getNativeFileName(directory + "/" + diskInfo['recurse'][i]);
                    if (top.secureDirectoryExists(new Function('return window'), recurseDirectory)) {
                        errorString = updateDiskMappings(recurseDirectory);                        
                    } else {
                        top.logMessage("LPV20032W", recurseDirectory);
                    }                    
                }
            } else {
                top.logMessage("LPV20031E", diskStructureFile);
            }
        } else {
            //check for disk label file and make the necessary updates
            var thisDiskPropFile = top.getNativeFileName(directory + "/" + top.RELATIVEDIR+top.DISKINFODIR+top.THISDISKINFO);
            if (top.secureFileExists(new Function('return window'), thisDiskPropFile)) {
                var diskLabelProperties = new Array();
                top.secureRead1PropertyFile(new Function('return window'), thisDiskPropFile, diskLabelProperties, false); 
                if (typeof diskLabelProperties.id != "undefined" && diskLabelProperties.id != top.diskLabel.id && isCompatibleDiskLabel(diskLabelProperties,top.diskLabel)) {
                    top.diskMappings[diskLabelProperties.id] = directory;
                } else {
                    if (typeof diskLabelProperties.id == "undefined" || diskLabelProperties.id == top.diskLabel.id) {
                        errorString = WRONG_DISK_ID;
                    } else {
                        errorString = getIncompatibleMessage(diskLabelProperties,top.diskLabel);
                    }
                }
            }
        }
    }
    catch(e) {
        top.logException(e,arguments);
    }
    return errorString;
}

//This is a helper function that looks for the top level of the media.  This is defined
//by the highest location of the file diskStructure.properties
//
//Things to fix: make getNativeFileName remove excess ../
//
//directory - the current directory that is being check for the existance of the diskStructure.properties file
//previousDirectory - the last directory that contains diskStructure.properties
function findMediaTop(directory, previousDirectory) {
    var mediaTop = previousDirectory;

    var parentDirIndex = directory.indexOf(".." + top.getNativeFileSeparator());
    if (parentDirIndex != -1) {
        directory = directory.substring(0, parentDirIndex);  //Once you get to the root, ../ start building up
    }                                                        //in the path and aren't removed by getNativeFileName
    if (previousDirectory != directory) {   //could get stuck in an infinite recursion at the media root without this check
        var file = top.getNativeFileName(directory + "/" + DISK_STRUCTURE_PROPERTIES);
        if (top.secureFileExists(new Function('return window'), file)) {
            mediaTop = findMediaTop(getNativeFileName(directory + "../"), directory);                  
        }
    }
    return mediaTop; 
}

//Scan for parent directories that may contain additional disk directories
var mediaTop = findMediaTop(top.getNativeFileName(top.STARTINGDIR + "../"), top.STARTINGDIR);

updateDiskMappings(mediaTop);

// read all valid labels
top.secureRead1PropertyFile(new Function('return window'), top.getNativeFileName(top.STARTINGDIR+top.RELATIVEDIR+top.DISKINFODIR+top.ALLDISKLABELS), top.allLabels, false);
// read this disk label
top.secureRead1PropertyFile(new Function('return window'), top.getNativeFileName(top.STARTINGDIR+top.RELATIVEDIR+top.DISKINFODIR+top.THISDISKINFO), top.diskLabel, false);
top.diskMappings[top.diskLabel.id] = top.STARTINGDIR;
// see if any sibling directories have valid disk labels that can be used so the user is not prompted as often
var parentdir = top.getNativeFileName(top.STARTINGDIR+"/../");
var dirs = top.secureGetDirectories(new Function('return window'), parentdir);
for (var i=0; i < dirs.length; i++) {
    var f = top.getNativeFileName(parentdir+dirs[i]+"/"+top.RELATIVEDIR+top.DISKINFODIR+top.THISDISKINFO);
    if (top.secureFileExists(new Function('return window'), f)) {
        var diskLabelProperties = new Array();
        top.secureRead1PropertyFile(new Function('return window'), f, diskLabelProperties, false);
        if (typeof diskLabelProperties.id != "undefined" && diskLabelProperties.id != top.diskLabel.id && isCompatibleDiskLabel(diskLabelProperties,top.diskLabel)) {
            top.diskMappings[diskLabelProperties.id] = parentdir+dirs[i];
        }
    }
}


